/* 16 Dec 2002. CLEANUP:  A utility for the DELTA IM module of SRE2003

  This will cleanup the DELTA cache directory of SRE2003.

************************************************/

/******* Begin user changable parameters *************************/

/* The delta directory. If '', then  the program will look for
   use the x:\sre2003\TEMP\DELTA directory.  */
delta_dir=''


/* the lifespan of an old instance, in days */
LIFESPAN=7

/* Maximum # of bytes of old "instances" to store (in megabyes).
   (hence, 50 means "50 megabytes) */
MAX_BYTES=50

/* Minimum amount of free space to retain on drive (in megabyes) */
MIN_BYTES_FREE=100



/******* End of user changable parameters *************************/

call init1              /* load some dlls */

cls
say  " " ; say

call lineout, bold cy_ye|| "CLEANUP: An SRE2003 utility  "normal
say
say "This program will cleanup (remove old entries) from the DELTA 'cache'"
say "directory. "
say "" 
say "   "bold"Delta cache directory: "normal||delta_dir
say
say "   "bold"             Max lifespan of cached items (in days): "normal||lifespan
say "   "bold"                    Maximum Mbytes allowed in cache: "normal||max_bytes
say "   "bold"Minimum Mbytes that must be available on this drive: "||normal||min_bytes_free
say
ii=yesno(" Is this correct",'No Yes')
if ii=0 then do
    say "You can change these parameters by editing "
    say "   " asource
    say "(look in the user configurable parameters at the top of the file)"
    exit
end


signal on novalue name foo1

say "Cleanup of of delta cache started at: "date('n')||' '||time('n')

call cleanup_task 

/* write time required */
a1=date('b') - 730000
a2=time('s')/86400
nowt2=a1+a2

tdif=nowt2-nowt
parse var tdif dys  '.' afrac
if afrac='' then afrac=0
if dys='' then dys=0 then
nsec=('.'||afrac)*86400

if dys>0 then
   say "(elapsed time: "dys" days  and " nsec " seconds)"
else
   say "(elapsed time: " nsec " seconds)"

exit

/*********************/
/* cleanup the delta_dir. Use the user-changeable parameters
   lifespan (days), max_bytes (M), min_bytes_free (M)
 */
cleanup_task:


delta_drive=left(delta_dir,1)

/* find all files in delta_dir */
foo=sysfiletree(delta_dir||'\*','FLIST','FT')
if flist.0=0 then do
  say  bold'No files in '||normal||delta_dir
  return
end
else do
   say bold||'# of files to be examined: 'normal||flist.0
end 

/* compute total size; and get file specific info */
gtot=0
say " Extracting information about these files ... "

do mm=1 to flist.0

  parse var flist.mm flist.mm.!date flist.mm.!size . flist.mm.!name
  fname=strip(flist.mm.!name)

  if left(flist.mm.!date,2)<80 then
        flist.mm.!date='20'||flist.mm.!date
  else
        flist.mm.!date='19'||flist.mm.!date

  gtot=gtot+strip(flist.mm.!size)

  foo=sysgetea(fname,'HOST','aserver')  /* this is used for "deletion marking*/
  flist.mm.!host=aserver
  flist.mm.!lastaccess=dosfileinfo(fname,'A')

  parse var flist.mm.!lastaccess month '/' day '/' year hour ':' minute ':' second
  if left(year,2)<80 then
     adate='20'||year
  else
     adate='19'||year
  bdate=strip(adate)||strip(month)||strip(day)
  cdate=dateconv(bdate,'S','B') - 730000
  atime=((hour*3600)+ (minute*60)+ second ) / 86400

  flist.mm.!lastaccess=cdate+atime 

 if mm//1000 =1 then do
      call charout,'0d'x|| "    "mm " of "flist.0 " files processed " 
  end
end
say
say " Total bytes in "flist.0" files = " ||addcomma(gtot)

/* first, clean up "old" files (mark for deletion, wait a minute, then delete */
todelete.0=0
say "Searching for files with last-access date more then "lifespan" days ago ..."
do jj=1 to flist.0
  if ( (nowt-flist.jj.!lastaccess) > lifespan ) | (flist.jj.!HOST='-1') then do
     aname=strip(flist.jj.!name)
     if flist.jj.!HOST<>-1 then foo=sysputea(aname,'HOST','-1')
     mn=todelete.0+1
     todelete.mn=strip(aname)
     todelete.mn.!size=flist.jj.!size
     todelete.0=mn
     flist.jj.!lastaccess=''    /* signal that it's been removed */
  end
end

totdel=0
diddelete=0

if todelete.0=0 then do
    say 'There are no too-old files to be deleted'
end 
else do
  say "  deleting " todelete.0 " files  (after pausing 10 seconds ...)"
  call syssleep(10)  /* wait a few seconds (on the off chance it's being used RIGHT now */
  do jj=1 to todelete.0
    a=sysfiledelete(todelete.jj)
    if a=0 then do
        diddelete=diddelete+1
        totdel=totdel+todelete.jj.!size
    end
  end
  say "     "diddelete" files, with "||addcomma(totdel)||" bytes, successfully deleted."
  gtot=gtot-totdel
end
diddelete0=diddelete

/* old files have been removed. Do we still need to delete more (too many bytes)? */
gtot0=gtot
isfree=dosdisk(delta_drive,'F')/1000000
gtot=gtot/1000000

NeedForMin=min_bytes_free-isfree
NeedForMax=gtot-max_bytes
mustDelete=max(needForMin,needForMax)
if mustDelete<0 then do
    say "Cache has been cleaned up. Remaining # of bytes="||addcomma(gtot0)
    return 1
end

/* if here, must delete mustDelete M bytes of files */

say "After deleting old files, "||addcomma(mustdelete)||' Mbytes must still be removed'
/* sort from oldest to newest */
alist.0=0
do jj=1 to flist.0
  if flist.jj.!lastaccess='' then iterate
  jj1=alist.0+1
  alist.jj1=left(flist.jj.!lastaccess,20)||flist.jj.!size' 'flist.jj.!name
  alist.0=jj1
end
say "Sorting "alist.0" files by last-access date ..."
foo=arraysort('ALIST',1,Alist.0,1,20,'A','N')
say "   files sorted, now readying files for deletion "
todelete.0=0
rtot=0
mustdelete=mustdelete*1000000
do jj=1 to alist.0
  parse var alist.jj . asize aname ; aname=strip(aname)
  foo=sysputea(aname,'HOST','-1')
  mn=todelete.0+1
  todelete.mn=aname
  todelete.0=mn
  rtot=rtot+strip(asize)
  if rtot>mustdelete then leave
end
say "   "||todelete.0 " files to be deleted (after pausing 10 seconds ...)"
call syssleep(10)  /* wait a minute (on the off chance it's being used RIGHT now */
diddelete=0
do jj=1 to todelete.0
  a=sysfiledelete(todelete.jj)
  if a=0 then do
        diddelete=diddelete+1
   end
end

say 'Expired files deleted= '||diddelete0||', additional files deleted= '||diddelete

return 1

foo1: say " novalue " sigl
exit
foo2: say " woow2 "sigl','rc
exit


/***************************/
/* initialize some stuff */
init1:

signal on error name foo2
signal on syntax name foo2
signal on failure name foo2

numeric digits 14


parse source . . asource

/* assuming this is run from x:\sre2003\bin\im\delta
  and delta cache dire is x:\sre2003\temp\delta
*/
asource=translate(strip(asource))
parse var asource dsource '\BIN\IM\DELTA' .
dsource=strip(strip(dsource),,'\')
def_delta_dir=strip(dsource)||'\TEMP\DELTA'

if delta_dir='' then delta_dir=def_delta_dir

foo=add_dllpath()

/* Load up advanced REXX functions */
call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
call SysLoadFuncs

crlf='0d0a'x

/*---   Load REXX libraries ----- */
/* Load up advanced REXX functions */
foo=rxfuncquery('sysloadfuncs')
if foo=1 then do
  call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
  call SysLoadFuncs
end
foo=rxfuncquery('rexxlibregister')
if foo=1 then do
 call rxfuncadd 'rexxlibregister','rexxlib', 'rexxlibregister'
 call rexxlibregister
end
foo=rxfuncquery('rexxlibregister')
if foo=1 then do
   say "Error: could not find REXXLIB.DLL (is it in your LIBPATH?) "
   exit
end 


  aesc='1B'x
  cy_ye=aesc||'[37;46;m'
  normal=aesc||'[0;m'
  bold=aesc||'[1;m'
  re_wh=aesc||'[31;47;m'
  reverse=aesc||'[7;m'

a1=date('b') - 730000
a2=time('s')/86400
nowt=a1+a2

return 0

/* -------------------- */
/* get a yes or no , return 1 if yes */
yesno0:procedure expose normal reverse bold
parse arg fooa , allopt
ayn='  '||bold||'Y'||normal||'es\'||bold||'N'||normal||'o'
do forever
 foo1=normal||reverse||fooa||normal||ayn
 call charout,  foo1 normal ':'
 pull anans
 if abbrev(anans,'Y')=1 then return 1
 if abbrev(anans,'N')=1 then return 0

end

/* -------------------- */
/* choose between 3 alternatives (by default,a yes or no ),
return 1 if yes (or 0,1,2 for chosen altenative ) */

yesno:procedure expose normal reverse bold cy_ye
parse arg amessage , altans,def,arrowok
ony2:
aynn=' '
if def='' then
 defans=''
else
 defans=translate(left(strip(def),1))
if altans='' then altans='No Yes'

w.0=words(altans)
goo=aynn
do iw0=1 to w.0
     w.iw0=strip(word(altans,iw0))
     a.iw0=translate(left(w.iw0,1))
     aa.iw0=substr(w.iw0,2)
     aynn=aynn||bold
     if  a.iw0=defans then aynn=aynn||cy_ye
     aynn=aynn||a.iw0||normal||aa.iw0
     goo=goo||a.iw0||aa.iw0
     if iw0<w.0 then do
       if noansi=0 then
          aynn=aynn'  '
       else
          aynn=aynn'  '
       goo=goo||'  '
     end
end
if arrowok=1 then aynn=aynn||' [UP]'

do forever
 foo1=normal||reverse||amessage||'? '||normal||aynn||': 'normal
 goo=amessage'?'||goo':'

 if length(goo)<73 then do
    call charout,foo1
 end
 else do
    foo1=normal||reverse||amessage||'? '||normal
    say foo1
    call charout,'     : 'aynn||': 'normal
 end

 anans=translate(sysgetkey('echo'))

 if anans='?' then do   /* asking for help */
     call online_help varname
     say
     signal ony2
 end 

 ianans=c2d(anans)
 if anans='' | ianans=13 | ianans=10 then  anans=defans


 do ijj=1 to w.0
    if abbrev(anans,a.ijj)=1 then do
        say
        return Ijj-1
    end
 end /* do */
 call charout,'0d'x
end


/*  add commas, and truncate decimal portion. */


/************/
/* ADD COMMAS TO A NUMBER */
addcomma:procedure
parse arg aval,ndec
parse var aval p1 '.' p2

if ndec='' then do
   p2=''
end
else do
   p2='.'||left(p2,ndec,'0')
end /* do */

plen=length(p1)
p1new=''
do i=1 to 10000 while plen>3
   p1new=','right(p1,3)||p1new
   p1=delstr(p1,plen-2)
   plen=plen-3
end /* do */

return p1||p1new||p2


/****************/
/* add x:\sre2003\dll to beginlibpath */
add_dllpath:procedure expose dsource
aa=rxqueue('S','SESSION')
do forever
 if queued()=0 then leave
 pull goo
end
address cmd '@SET BEGINLIBPATH  | rxqueue '
goo=''
if queued() then do
  pull goo
end 
parse var goo . '=' goo 
goo=strip(goo)
aa=rxqueue('S',aa)
dlldir=dsource||'\DLL'
foo=dlldir||';'||goo
address cmd '@SET BEGINLIBPATH = 'foo

return 1

