/* This is will undifference a file */
parse arg f1 f2
if f1='' then do 
    say "A unGNUdiff utility. "
    say "Syntax:  x:GnUnDiff old_file dif_file > newfile "
    say "         where dif_File was created using diff -e "
    exit
end 

foo=gnu_undiff(f1,f2)
call charout,foo
exit

/* undifference a file, using a difference file produced
  by gnudiff -e .
This should work for almost all such difference files. However, there
may be cases where more complicated "ed"  commands are used gnudiff --
in which case an error is returned */

gnu_undiff:
parse arg oldfile,diffile

oldstuff=get_file(oldfile)
if oldstuff='' then do
    say "Error: no such oldfile = "oldfile
    return ' '
end 

diffstuff=get_file(diffile)
if diffstuff='' then do
    say "Error: no such difference file = "diffile
    return ' '
end 

maxlen=length(diffstuff)+length(oldstuff)

/* parse oldstuff into an array */
iat=1
i0=0
stuff.=''
do forever
   i0=i0+1
   iat1=pos('0d0a'x,oldstuff,iat)
   if iat1=0 then do
     stuff.i0=substr(oldstuff,iat)
     leave
   end
   if iat1=iat then do
      stuff.i0=''
   end
   else do
      stuff.i0=substr(oldstuff,iat,iat1-iat)
   end
   iat=iat1+2
end 
stuff.0=i0

/* in general, dif files are written bottom to top. But not always.
Dif file structure is:
  n1[,n2]x stuff
  moresstuff
  .
where
  n1: line number. Changes occur AFTER this line. n1=0 means "change first line"
  [,n2] : optional "end" line number. For removing multiple lines
  x:  a,c,d, or s.  
        a = add stuff and morestuf
        c =change lines (remove and replace)
        d = delete lines
        s = substitute. Only used for single . on a line
  stuff.. morestuff: multiple lines of substitution
  .     end of this change block
Note that a change block can never conists of a  single . in character 1

*/

lastline=stuff.0+1
do forever
    if diffstuff='' then do
        leave
   end
    parse var diffstuff acmd '0d0a'x diffstuff
    ichar=verify(acmd,'0123456789,.')            /* acd or s */
    if ichar='' | ichar=0 then do
       say "Error: unrecognized diff command: "||acmd
      return ' '
    end
    bcmd=substr(acmd,ichar,1)
    if pos(bcmd,'acds')=0 then do
         say "Error: unrecognized ED option= "bcmd
         return ' '
    end
    ccmd=substr(acmd,ichar+1)
    rnge=left(acmd,ichar-1)
    parse var rnge line1 ',' line2
    if line2='' then line2=line1
    if datatype(line1)<>'NUM' & datatype(line2)<>'NUM' then do
        say "Bad line range: "rnge
        return ''
    end
    if line1>line2 then do
        say "Misordered line range: "line1 ' - ' line2
        return ''
    end

    if line1> lastline then do         /* darn, have to regenerate "Stuff" */
       say " regenerating ..."
       foo=regen_stuff(maxlen)           /* since might be changing "changed" lines */
    end 
   
    select
       when bcmd='d' then do
          do jj=line1 to line2
              stuff.jj.!del=1
          end 
       end

       when bcmd='c'  then do
          do jj=line1+1 to line2        /* line1 is changed, with possible embedded crlfs */
              stuff.jj.!del=1           /* others are deleted */
          end 
          ff=''
          do forever                    /* stop when  "." is found */
             if length(diffstuff)=0 then do
                  say "Error no terminating ' . ' "
                  return ' '
              end
              parse var diffstuff getme '0d0a'x diffstuff
              if getme='.' then leave    /* got the replacement */
              if length(ff)<>0 then ff=ff||'0d0a'x
              ff=ff||getme   
          end 
          stuff.line1.!del=0 
          stuff.line1=ff
       end

       when bcmd='a'  then do
          line1a=line1+1
          ff='' ; igot=0
          do forever                    /* stop when  "." is found */
             if length(diffstuff)=0 then do
                  say "Error no terminating ' . ' "
                  return ' '
              end
              parse var diffstuff getme '0d0a'x diffstuff
              if getme='.' then leave    /* got the replacement */
              if igot=1 then ff=ff||'0d0a'x
              igot=1
              ff=ff||getme   
          end 
          if stuff.line1a.!del=1 then do
             stuff.line1a=ff
          end
          else do
             stuff.line1a=ff||'0d0a'x||stuff.line1a
          end
          stuff.line1a.!del=0
          if stuff.0<line1a  then stuff.0=line1a
       end           

       when bcmd='s' then do
         if ccmd<>'/^\.\././' then do
                say "Unrecognized s command: "ccmd
                return ' '
         end 
         stuff.line1='.'
         stuff.line1.!del=0 
       end 
       otherwise do
          say "Unrecognized command: "acmd
          return ' '
       end
    end  /* select */
   lastline=line1              /* the smallest line# changed */
end 

nustuff=''
igot=0
do az=1 to stuff.0
   if stuff.az.!del=1 then iterate
   if igot=1 then
     nustuff=nustuff||'0d0a'x||stuff.az
   else
     nustuff=stuff.az
   igot=1

end
if left(nustuff,1)<>'1a'x then nustuff=nustuff||'0d0a'x
return nustuff


/*****/
/* regenerate stuff */
regen_stuff:procedure expose stuff. 

t0=0
do az=1 to stuff.0
   if stuff.az.!del=1 then iterate
   taa=stuff.az
   do forever
      t0=t0+1
      iarf=pos('0d0a'x,taa)
      if iarf=0 then do
         newt.t0=taa
         leave
      end
      if iarf=1 then do
         newt.t0=''
      end 
      newt.t0=left(taa,iarf-1)
      taa=substr(taa,iarf+2)
   end
end
stuff.=''
do mm=1 to t0
   stuff.mm=newt.mm
end 
stuff.0=t0

return 1

/*************************/
get_file:procedure
parse arg aname

aname=strip(aname)
if stream(aname,'c','query exists')='' then return ''

foo=stream(aname,'c','open read')
if abbrev(translate(foo),'READY')=0 then return ''

ii=stream(aname,'c','query size')
if ii='' | ii=0 then return ''

stuff=charin(aname,1,ii)
foo=stream(aname,'c','close')
return stuff


